<?php

declare(strict_types=1);

namespace Dedi\SyliusMailAdminPlugin\EventListener\Mail;

use Dedi\SyliusMailAdminPlugin\Repository\MailRepositoryInterface;
use Sylius\Bundle\CoreBundle\Mailer\Emails as CoreBundleEmails;
use Sylius\Bundle\UserBundle\Mailer\Emails as UserBundleEmails;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\ShopUserInterface;
use Sylius\Component\Locale\Context\LocaleContextInterface;
use Sylius\Component\Mailer\Sender\SenderInterface;
use Sylius\Component\User\Model\UserInterface;
use Symfony\Component\EventDispatcher\GenericEvent;
use Webmozart\Assert\Assert;

class MailerListener
{
    public function __construct(
        private readonly SenderInterface $emailSender,
        private readonly ChannelContextInterface $channelContext,
        private readonly LocaleContextInterface $localeContext,
        private readonly MailRepositoryInterface $mailRepository,
    ) {
    }

    public function sendResetPasswordTokenEmail(GenericEvent $event): void
    {
        $mails = $this->mailRepository->findByCode(UserBundleEmails::RESET_PASSWORD_TOKEN);

        foreach ($mails as $mail) {
            if ($mail->isEnabled()) {
                $this->sendEmail($event->getSubject(), UserBundleEmails::RESET_PASSWORD_TOKEN);
            }
        }
    }

    public function sendResetPasswordPinEmail(GenericEvent $event): void
    {
        $mails = $this->mailRepository->findByCode(UserBundleEmails::RESET_PASSWORD_PIN);

        foreach ($mails as $mail) {
            if ($mail->isEnabled()) {
                $this->sendEmail($event->getSubject(), UserBundleEmails::RESET_PASSWORD_PIN);
            }
        }
    }

    public function sendVerificationTokenEmail(GenericEvent $event): void
    {
        $mails = $this->mailRepository->findByCode(UserBundleEmails::EMAIL_VERIFICATION_TOKEN);

        foreach ($mails as $mail) {
            if ($mail->isEnabled()) {
                $this->sendEmail($event->getSubject(), UserBundleEmails::EMAIL_VERIFICATION_TOKEN);
            }
        }
    }

    public function sendUserRegistrationEmail(GenericEvent $event): void
    {
        $customer = $event->getSubject();

        Assert::isInstanceOf($customer, CustomerInterface::class);

        $user = $customer->getUser();
        if (null === $user) {
            return;
        }

        $email = $customer->getEmail();
        if ($email === null) {
            return;
        }

        Assert::isInstanceOf($user, ShopUserInterface::class);

        $mails = $this->mailRepository->findByCode(CoreBundleEmails::USER_REGISTRATION);

        foreach ($mails as $mail) {
            if ($mail->isEnabled()) {
                $this->sendEmail($user, CoreBundleEmails::USER_REGISTRATION);
            }
        }
    }

    private function sendEmail(UserInterface $user, string $emailCode): void
    {
        $email = $user->getEmail();
        Assert::notNull($email);

        $this->emailSender->send(
            $emailCode,
            [$email],
            [
                'user' => $user,
                'channel' => $this->channelContext->getChannel(),
                'localeCode' => $this->localeContext->getLocaleCode(),
            ],
        );
    }
}
