<?php

declare(strict_types=1);

namespace Dedi\SyliusMailAdminPlugin\EventListener\Mail;

use Dedi\SyliusMailAdminPlugin\Repository\MailRepositoryInterface;
use Sylius\Bundle\CoreBundle\Mailer\Emails;
use Sylius\Bundle\CoreBundle\Mailer\OrderEmailManagerInterface;
use Sylius\Component\Core\Model\CustomerInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Mailer\Sender\SenderInterface;
use Webmozart\Assert\Assert;

class OrderEmailManager implements OrderEmailManagerInterface
{
    public function __construct(
        private SenderInterface $emailSender,
        private MailRepositoryInterface $mailRepository,
    ) {
    }

    public function sendConfirmationEmail(OrderInterface $order): void
    {
        /** @var CustomerInterface $customer */
        $customer = $order->getCustomer();
        $email = $customer->getEmail();
        Assert::notNull($email);

        $mails = $this->mailRepository->findByCode(Emails::ORDER_CONFIRMATION);

        foreach ($mails as $mail) {
            if ($mail->isEnabled()) {
                $this->emailSender->send(
                    Emails::ORDER_CONFIRMATION,
                    [$email],
                    [
                        'order' => $order,
                        'channel' => $order->getChannel(),
                        'localeCode' => $order->getLocaleCode(),
                    ],
                );
            }
        }
    }
}
